using System;
using System.Collections.Generic;
using System.Text;

namespace InfiniTec.Threading
{
    /// <summary>
    /// This operation action causes the specified operation to be executed. 
    /// </summary>
    public class NestedOperation: NestedOperationStep
    {
        private bool _PropagateProgressToRootOperation;
        private bool _PropagateTimeout;

        /// <summary>
        /// True, if a timeout is propagated up the call chain.
        /// False, otherwise.
        /// </summary>
        /// <remarks>
        /// If set to false, the calling operation must check the result
        /// status of the nested operation, whether the operation has
        /// completed successfully.
        /// </remarks>
        public bool PropagateTimeout
        {
            get { return _PropagateTimeout; }
            set { _PropagateTimeout = value; }
        }

        /// <summary>
        /// If set to true, progress updates are propagated to the root operation. 
        /// </summary>
        public bool PropagateProgressToRootOperation
        {
            get { return _PropagateProgressToRootOperation; }
            set { _PropagateProgressToRootOperation = value; }
        }

        /// <summary>
        /// Initiates a new instance of this class using the specified values.
        /// </summary>
        /// <param name="nestedOperation">The <see cref="AsyncOperation"/> to execute.</param>
        /// <param name="propagateProgressToRootOperation">If set to true, progress updates are propagated to the root operation.</param>
        /// <param name="exceptionsToCatch">The types of <see cref="Exception"/> handled by the calling operation.</param>
        public NestedOperation(AsyncOperation nestedOperation, bool propagateProgressToRootOperation,
            params Type[] exceptionsToCatch): base(nestedOperation, exceptionsToCatch)
        {
            _PropagateTimeout = true;
            PropagateProgressToRootOperation = propagateProgressToRootOperation;
        }
    }
}
