using System;
using System.Collections.Generic;
using System.Text;

namespace InfiniTec.Threading
{
    /// <summary>
    /// This operation action causes the specified operation to be executed. Additionally, a progress update is 
    /// returned.
    /// </summary>
    /// <typeparam name="StatusType"></typeparam>
    public class NestedOperation<StatusType>: NestedOperation
    {
        private ProgressUpdate<StatusType> _Status;

        /// <summary>
        /// The new state of the current operation.
        /// </summary>
        public ProgressUpdate<StatusType> Status
        {
            get { return _Status; }
        }

        /// <summary>
        /// Initiates a new instance of this class using the specified values.
        /// </summary>
        /// <param name="nestedOperation">The <see cref="AsyncOperation"/> to execute.</param>
        /// <param name="propagateProgressToRootOperation">If set to true, progress updates are propagated to the root operation.</param>
        /// <param name="exceptionsToCatch">The types of <see cref="Exception"/> handled by the calling operation.</param>
        /// <param name="status">The new state of the current operation.</param>
        public NestedOperation(AsyncOperation nestedOperation, bool propagateProgressToRootOperation,
            ProgressUpdate<StatusType> status, params Type[] exceptionsToCatch)
            : base(nestedOperation, propagateProgressToRootOperation, exceptionsToCatch)
        {
            _Status = status;
        }
    }
}
