using System;
using System.Collections.Generic;
using System.Text;

namespace InfiniTec.Threading
{
    /// <summary>
    /// Allows an asynchronous operation process several nested operations in paralell.
    /// </summary>
    public class ParallelOperations: OperationAction
    {
        private IEnumerable<NestedOperation> _Operations;
        private int _MaxConcurrentOperations;

        /// <summary>
        /// Gets the operations to be executed in paralell.
        /// </summary>
        /// <value>The operations.</value>
        public IEnumerable<NestedOperation> Operations
        {
            get { return _Operations; }
        }

        /// <summary>
        /// Gets the maximum number concurrent operations.
        /// </summary>
        /// <value>The max concurrent operations.</value>
        public int MaxConcurrentOperations
        {
            get { return _MaxConcurrentOperations; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParallelOperations"/> class.
        /// </summary>
        /// <param name="operations">The operations to execute.</param>
        /// <param name="maxConcurrentOperations">The maximum number of  current operations.</param>
        public ParallelOperations(IEnumerable<NestedOperation> operations, int maxConcurrentOperations)
        {
            _Operations = operations;
            _MaxConcurrentOperations = maxConcurrentOperations;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParallelOperations"/> class.
        /// </summary>
        /// <param name="operations">The operations to execute</param>
        public ParallelOperations(IEnumerable<NestedOperation> operations) : this(operations, int.MaxValue) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParallelOperations"/> class.
        /// </summary>
        /// <param name="operations">The operations to process.</param>
        public ParallelOperations(params NestedOperation[] operations) : this((IEnumerable<NestedOperation>) operations) {}
        /// <summary>
        /// Initializes a new instance of the <see cref="ParallelOperations"/> class.
        /// </summary>
        /// <param name="maxConcurrentOperations">The maximum number of concurrent operations.</param>
        /// <param name="operations">The operations.</param>
        public ParallelOperations(int maxConcurrentOperations, params NestedOperation[] operations) : this((IEnumerable<NestedOperation>)operations, maxConcurrentOperations) { }
    }
}
