using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace InfiniTec.Threading
{
    /// <summary>
    /// Abstract base class for progress updates.
    /// </summary>
    public abstract class ProgressUpdate: OperationAction
    {
        private WaitHandle _AsyncWaitHandle;

        /// <summary>
        /// Gets or sets the <see cref="WaitHandle"/> to wait for.
        /// </summary>
        public WaitHandle AsyncWaitHandle
        {
            get { return _AsyncWaitHandle; }
            set { _AsyncWaitHandle = value; }
        }

        private int _ProgressPercentage;

        /// <summary>
        /// Gets or sets the current progress in percent.
        /// </summary>
        public int ProgressPercentage
        {
            get { return _ProgressPercentage; }
            set
            {
                if (value < 0 || value > 100) throw new ArgumentOutOfRangeException("ProgressPercentage", "ProgressPercentage must be between 0 and 100");
                _ProgressPercentage = value;
            }
        }

        /// <summary>
        /// Initiates a new instance of this class and sets the progress and waithandle to the specified value.
        /// </summary>
        /// <param name="progressPercentage">The progress of the operation.</param>
        /// <param name="asyncWaitHandle">The waithandle to wait for.</param>
        protected ProgressUpdate(int progressPercentage, WaitHandle asyncWaitHandle)
            : base()
        {
            _AsyncWaitHandle = asyncWaitHandle;
            _ProgressPercentage = progressPercentage;
        }

        /// <summary>
        /// Initiates a new instance of this class.
        /// </summary>
        protected ProgressUpdate()
            : base()
        {
        }
    }

}
