using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace InfiniTec.Threading
{
    /// <summary>
    /// The method implements some delegate helper functions.
    /// </summary>
    public static class DelegateHelper
    {
        /// <summary>
        /// Calls the specified delegate. If the target object of the delegate (or delegates in case of a <see cref="MulticastDelegate"/>)
        /// implements the <see cref="ISynchronizeInvoke"/> interface, this interface is used to call the delegate.
        /// Otherwise, the delegate is called directly.
        /// </summary>
        /// <param name="target">The delegate to call</param>
        /// <param name="parameters">The parameters to pass to the delegate</param>
        /// <remarks>To modify controls on a windows forms, those modifications must be made from the same
        /// thread which created the controls in the first place. <see cref="ISynchronizeInvoke"/>
        /// is used here to execeute the delegate on that specific thread.</remarks>
        public static void SynchronizedDelegateInvoke(Delegate target, params object[] parameters)
        {
            if (target == null) throw new ArgumentNullException("target must not be null", "target");

            foreach (Delegate t in target.GetInvocationList())
            {
                if (t.Target is ISynchronizeInvoke)
                {
                    ((ISynchronizeInvoke)t.Target).Invoke(t, parameters);
                }
                else target.DynamicInvoke(parameters);
            }
        }

    }
}
